@@ -41,21 +41,55 @@
 #include "../config.h"
 
 #include "chat_window.h"
-#include "ldap_window.h"
 #include "gnomemeeting.h"
-#include "ldap_window.h"
 #include "callbacks.h"
 #include "misc.h"
-#include "menu.h"
-#include "callbacks.h"
+#include "main_window.h"
 
+#include "gm_conf.h"
 #include "gtk-text-tag-addon.h"
 #include "gtk-text-buffer-addon.h"
 #include "gtk-text-view-addon.h"
 #include "gtk_menu_extensions.h"
 
-extern GtkWidget *gm;
 
+/* internal structure used by the text chat */
+typedef struct _GmTextChat
+{
+  GtkWidget     *text_view;
+  GtkTextBuffer *text_buffer;
+  gboolean	something_typed;
+  gchar		*begin_msg;
+} GmTextChat;
+
+
+/* declaration of the static functions */
+static void gm_text_chat_destroy (gpointer chat_pointer);
+static void copy_uri_callback (const gchar *uri);
+static void connect_uri_callback (const gchar *uri);
+static void add_uri_callback (const gchar *uri);
+static void chat_entry_activate (GtkEditable *w, gpointer data);
+#ifndef DISABLE_GNOME
+static void open_uri_callback (const gchar *uri);
+#endif
+
+/* DESCRIPTION  :  Called when the chat window is destroyed
+ * BEHAVIOR     :  Frees the GmTextChat* that is embedded in the window
+ * PRE          :  /
+ */
+static void
+gm_text_chat_destroy (gpointer chat_pointer)
+{
+  GmTextChat *chat = (GmTextChat *)chat_pointer;
+
+  g_return_if_fail (chat != NULL);
+  
+  if (chat->begin_msg != NULL) {
+     g_free (chat->begin_msg);
+     chat->begin_msg = NULL;
+  }
+  delete chat;
+}
 
 #ifndef DISABLE_GNOME
 /* DESCRIPTION  :  Called when an URL is clicked.
@@ -65,8 +99,9 @@ extern GtkWidget *gm;
 static void
 open_uri_callback (const gchar *uri)
 {
-  if (uri)
-    gnome_url_show (uri, NULL);
+  g_return_if_fail (uri != NULL);
+  
+  gnome_url_show (uri, NULL);
 }
 #endif
 
@@ -78,12 +113,12 @@ open_uri_callback (const gchar *uri)
 static void
 copy_uri_callback (const gchar *uri)
 {
-  if (uri) {
-    gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-			    uri, -1);
-    gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
-			    uri, -1);
-  }
+  g_return_if_fail (uri != NULL);
+
+  gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+			  uri, -1);
+  gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
+			  uri, -1);
 }
 
 
@@ -94,26 +129,20 @@ copy_uri_callback (const gchar *uri)
 static void
 connect_uri_callback (const gchar *uri)
 {
+  GtkWidget *main_window = NULL;
   GMH323EndPoint *ep = NULL;
-  GmWindow *gw = NULL;
   
-  ep = GnomeMeeting::Process ()->Endpoint ();
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
-    
-  if (uri) {
+  g_return_if_fail (uri != NULL);
 
-    if (ep->GetCallingState () == GMH323EndPoint::Standby) {
+  main_window = GnomeMeeting::Process ()->GetMainWindow ();
+  ep =  GnomeMeeting::Process ()->Endpoint ();
 
-      gw = GnomeMeeting::Process ()->GetMainWindow ();
-      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry),
-			  uri);
-      
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gw->connect_button),
-				    true);
-    }
-    else if (ep->GetCallingState () == GMH323EndPoint::Connected)
-      transfer_call_cb (NULL, (gpointer) uri);
+  if (ep->GetCallingState () == GMH323EndPoint::Standby) {    
+
+    GnomeMeeting::Process ()->Connect (uri);
   }
+  else if (ep->GetCallingState () == GMH323EndPoint::Connected)
+    gm_main_window_transfer_dialog_run (main_window, (gchar *) uri);
 }
 
 
@@ -124,8 +153,10 @@ connect_uri_callback (const gchar *uri)
 static void
 add_uri_callback (const gchar *uri)
 {
-  if (uri)
-    gnomemeeting_addressbook_edit_contact_dialog ((gchar *) uri);
+  g_return_if_fail (uri != NULL);
+
+  g_warning ("FIX ME: Not implemented yet");
+  //gnomemeeting_addressbook_edit_contact_dialog ((gchar *) uri);
 }  
 
 
@@ -138,6 +169,7 @@ chat_entry_activate (GtkEditable *w,
 		     gpointer data)
 {
   GMH323EndPoint *endpoint = GnomeMeeting::Process ()->Endpoint ();
+  GtkWidget *chat_window = GnomeMeeting::Process ()->GetChatWindow ();
   PString s;
     
   if (endpoint) {
@@ -152,58 +184,45 @@ chat_entry_activate (GtkEditable *w,
     if (endpoint->GetCallingState () == GMH323EndPoint::Connected
 	&& !s.IsEmpty ()) {
             
-      /* If the GDK lock is taken, the connection will never get a
-	 chance to be established if we lock its mutex */
+      /* Don't hold any lock when asking things to the endpoint */
       gdk_threads_leave ();
-      H323Connection *connection = 
-	endpoint->FindConnectionWithLock (endpoint->GetCurrentCallToken ());
+      endpoint->SendTextMessage (endpoint->GetCurrentCallToken (), s);
       gdk_threads_enter ();
 
-      if (connection != NULL)  {
-                
-	connection->SendUserInput ("MSG"+s);
-
-	if (g_utf8_validate ((gchar *) (const unsigned char*) local, -1, NULL))
-	  utf8_local = g_strdup ((char *) (const char *) (local));
-	else
-	  utf8_local = gnomemeeting_from_iso88591_to_utf8 (local);
-
-	if (utf8_local)
-	  gnomemeeting_text_chat_insert (utf8_local, s, 0);
-	g_free (utf8_local);
-                
-	gtk_entry_set_text (GTK_ENTRY (w), "");
-
-	connection->Unlock ();
-      }
+      if (g_utf8_validate ((gchar *) (const unsigned char*) local, -1, NULL))
+	utf8_local = g_strdup ((char *) (const char *) (local));
+      else
+	utf8_local = gnomemeeting_from_iso88591_to_utf8 (local);
+      
+      if (utf8_local)
+	gnomemeeting_text_chat_insert (chat_window, utf8_local, s, 0);
+      g_free (utf8_local);
+      
+      gtk_entry_set_text (GTK_ENTRY (w), "");
     }
   }
 }
 
-void gnomemeeting_text_chat_clear (GtkWidget *w,
-				   GmTextChat *chat)
+void gnomemeeting_text_chat_clear (GtkWidget *chat_window)
 {
-  GmWindow *gw = NULL;
+  GmTextChat *chat = NULL;
   GtkTextIter start_iter, end_iter;
 
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  g_return_if_fail (chat_window != NULL);
   
+  chat = (GmTextChat *)g_object_get_data (G_OBJECT (chat_window), "GMObject");
   gtk_text_buffer_get_start_iter (chat->text_buffer, &start_iter);
   gtk_text_buffer_get_end_iter (chat->text_buffer, &end_iter);
 
   gtk_text_buffer_delete (chat->text_buffer, &start_iter, &end_iter);
-
-  gtk_menu_set_sensitive (gw->main_menu, "clear_text_chat", FALSE);
 }
 
 void
-gnomemeeting_text_chat_call_start_notification (void)
+gnomemeeting_text_chat_call_start_notification (GtkWidget *chat_window)
 {
   GmTextChat *chat = NULL;
-  GmWindow *gw = NULL;
 
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
-  chat = GnomeMeeting::Process ()->GetTextChat ();
+  g_return_if_fail (chat_window != NULL);
 	
   // find the time at which the event occured
   time_t *timeptr;
@@ -216,65 +235,71 @@ gnomemeeting_text_chat_call_start_notification (void)
   strftime(time_str, 20, "%H:%M:%S", localtime (timeptr));
 
   // prepare the message
+  chat = (GmTextChat *)g_object_get_data (G_OBJECT (chat_window), "GMObject");
   if (chat->begin_msg) g_free (chat->begin_msg); // shouldn't happen...
-  chat->begin_msg = g_strdup_printf ("---- Call begins at %s\n", time_str);
+  chat->begin_msg = g_strdup_printf (_("---- Call begins at %s\n"), time_str);
 
   // we are ready to trigger the message display
   chat->something_typed = FALSE;
   
   //  free what we should
-  g_free (time_str);
+  free (time_str);
 }
 
 void
-gnomemeeting_text_chat_call_stop_notification (void)
+gnomemeeting_text_chat_call_stop_notification (GtkWidget *chat_window)
 {
   GtkTextIter iter;
   GmTextChat *chat = NULL;
-  GmWindow *gw = NULL;
   gchar *text = NULL;
+  gboolean auto_clear = FALSE;
 
-  // find the time at which the event occured	
-  time_t *timeptr;
-  char *time_str;
-  
-  time_str = (char *) malloc (21);
-  timeptr = new (time_t);
- 
-  time (timeptr);
-  strftime(time_str, 20, "%H:%M:%S", localtime (timeptr));
+  g_return_if_fail (chat_window != NULL);
 
-  // prepare the message to be displayed
-  text = g_strdup_printf ("---- Call ends at %s\n", time_str);
+  auto_clear = gm_conf_get_bool (USER_INTERFACE_KEY "auto_clear_text_chat");
 
-  // displays the message
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
-  chat = GnomeMeeting::Process ()->GetTextChat ();
-  gtk_text_buffer_get_end_iter (chat->text_buffer, &iter);
+  if (auto_clear)
+    gnomemeeting_text_chat_clear (chat_window);
+  else {
+    // find the time at which the event occured	
+    time_t *timeptr;
+    char *time_str;
+    
+    time_str = (char *) malloc (21);
+    timeptr = new (time_t);
+    
+    time (timeptr);
+    strftime(time_str, 20, "%H:%M:%S", localtime (timeptr));
+    
+    // prepare the message to be displayed
+    text = g_strdup_printf (_("---- Call ends at %s\n"), time_str);
+    
+    // displays the message
+    chat = (GmTextChat *)g_object_get_data (G_OBJECT (chat_window), 
+					    "GMObject");
+    gtk_text_buffer_get_end_iter (chat->text_buffer, &iter);
+    
+    if (chat->something_typed == TRUE)
+      gtk_text_buffer_insert(chat->text_buffer, &iter, text, -1);
 
-  if (chat->something_typed == TRUE)
-    gtk_text_buffer_insert(chat->text_buffer, &iter, text, -1);
-
-  // freeing what we need to
-  free (time_str);
-  g_free (text);
+    // freeing what we need to
+    free (time_str);
+    g_free (text);
+  }
 }
 
 void 
-gnomemeeting_text_chat_insert (PString local,
-			       PString str,
-			       int user)
+gnomemeeting_text_chat_insert (GtkWidget *chat_window, PString name,
+			       PString str, int user)
 {
   gchar *msg = NULL;
   GtkTextIter iter;
   GtkTextMark *mark;
-  
   GmTextChat *chat = NULL;
-  GmWindow *gw = NULL;
 
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
-  chat = GnomeMeeting::Process ()->GetTextChat ();
+  g_return_if_fail (chat_window != NULL);
 
+  chat = (GmTextChat *)g_object_get_data (G_OBJECT (chat_window), "GMObject");
   gtk_text_buffer_get_end_iter (chat->text_buffer, &iter);
 
   // delayed call begin notification first!
@@ -287,7 +312,7 @@ gnomemeeting_text_chat_insert (PString local,
     }
   }
 
-  msg = g_strdup_printf ("%s: ", (const char *) local);
+  msg = g_strdup_printf ("%s: ", (const char *) name);
 
   if (user == 1)
     gtk_text_buffer_insert_with_tags_by_name (chat->text_buffer, &iter, msg, 
@@ -307,13 +332,11 @@ gnomemeeting_text_chat_insert (PString local,
 
   gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (chat->text_view), mark, 
 				0.0, FALSE, 0,0);
-
-  gtk_menu_set_sensitive (gw->main_menu, "clear_text_chat", TRUE);
 }
 
 
 GtkWidget *
-gnomemeeting_text_chat_new (GmTextChat *chat)
+gnomemeeting_text_chat_new ()
 {
   GtkWidget *entry = NULL;
   GtkWidget *scr = NULL;
@@ -322,14 +345,19 @@ gnomemeeting_text_chat_new (GmTextChat *chat)
   GtkWidget *frame = NULL;
   GtkWidget *hbox = NULL;
   GtkWidget *chat_window = NULL;
-
-  GtkTextIter  iter;
+  GmTextChat *chat = NULL;
+  GtkTextIter iter;
   GtkTextMark *mark = NULL;
   GtkTextTag *regex_tag = NULL;
 
 
   /* Get the structs from the application */
+  chat = new GmTextChat ();
   chat_window = gtk_frame_new (NULL);
+  g_object_set_data_full (G_OBJECT (chat_window), "GMObject",
+                          (gpointer) chat,
+			  (GDestroyNotify) (gm_text_chat_destroy));
+
   gtk_frame_set_shadow_type (GTK_FRAME (chat_window), GTK_SHADOW_NONE);
   table = gtk_table_new (1, 3, FALSE);
   
@@ -374,7 +402,7 @@ gnomemeeting_text_chat_new (GmTextChat *chat)
 					  "foreground", "blue",
 					  NULL);
   if (gtk_text_tag_set_regex (regex_tag,
-			      "\\<([s]?(ht|f)tp://[^[:blank:]]+)\\>")) {
+			      "\\<(http[s]?|[s]?ftp)://[^[:blank:]]+\\>")) {
     gtk_text_tag_add_actions_to_regex (regex_tag,
 #ifndef DISABLE_GNOME
 				       _("Open URI"),
