@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,9 +30,9 @@
  *                         gatekeeper.cpp  -  description
  *                         ------------------------------
  *   begin                : Wed Sep 19 2001
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : Multithreaded class to register to gatekeepers
- *                          given the options in gconf.
+ *                          given the options in config.
  *
  */
 
@@ -41,55 +41,54 @@
 
 #include "gatekeeper.h"
 #include "gnomemeeting.h"
+#include "main_window.h"
 #include "misc.h"
+#include "log_window.h"
 
 #include "dialog.h"
-#include "gconf_widgets_extensions.h"
+#include "gm_conf.h"
 
 #include <h323pdu.h>
 
 
-/* Declarations */
-extern GtkWidget *gm;
-
 /* The class */
 GMH323Gatekeeper::GMH323Gatekeeper ()
-  :PThread (1000, AutoDeleteThread)
+  :PThread (1000, NoAutoDeleteThread)
 {
-  gchar *gconf_string = NULL;
+  gchar *conf_string = NULL;
     
-  /* Query the gconf database for options */
+  /* Query the config database for options */
   gnomemeeting_threads_enter ();
   registering_method =
-    gconf_get_int (H323_GATEKEEPER_KEY "registering_method");
+    gm_conf_get_int (H323_GATEKEEPER_KEY "registering_method");
 
   /* Gatekeeper password */
-  gconf_string = gconf_get_string (H323_GATEKEEPER_KEY "password");
-  if (gconf_string) {
+  conf_string = gm_conf_get_string (H323_GATEKEEPER_KEY "password");
+  if (conf_string) {
     
-    gk_password = PString (gconf_string);
-    g_free (gconf_string);
+    gk_password = PString (conf_string);
+    g_free (conf_string);
   }
 
   /* Gatekeeper host */
   if (registering_method == 1) {
     
-    gconf_string = gconf_get_string (H323_GATEKEEPER_KEY "host");
-    if (gconf_string) {
+    conf_string = gm_conf_get_string (H323_GATEKEEPER_KEY "host");
+    if (conf_string) {
       
-      gk_host = PString (gconf_string);
-      g_free (gconf_string);
+      gk_host = PString (conf_string);
+      g_free (conf_string);
     }
   }
 
   /* Gatekeeper ID */
   if (registering_method == 2) {
     
-    gconf_string = gconf_get_string (H323_GATEKEEPER_KEY "id");
-    if (gconf_string) {
+    conf_string = gm_conf_get_string (H323_GATEKEEPER_KEY "id");
+    if (conf_string) {
       
-      gk_id = PString (gconf_string);
-      g_free (gconf_string);
+      gk_id = PString (conf_string);
+      g_free (conf_string);
     }
   }
   gnomemeeting_threads_leave ();
@@ -101,11 +100,15 @@ GMH323Gatekeeper::GMH323Gatekeeper ()
 GMH323Gatekeeper::~GMH323Gatekeeper ()
 {
   /* Nothing to do here */
+  PWaitAndSignal m(quit_mutex);
 }
 
 
 void GMH323Gatekeeper::Main ()
 {
+  GtkWidget *main_window = NULL;
+  GtkWidget *history_window = NULL;
+
   PString gk_name;
   gchar *msg = NULL;
 
@@ -113,25 +116,27 @@ void GMH323Gatekeeper::Main ()
   H323Gatekeeper *gatekeeper = NULL;
   
   BOOL no_error = TRUE;
-  GmWindow *gw = NULL;
   
   endpoint = GnomeMeeting::Process ()->Endpoint ();
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  main_window = GnomeMeeting::Process ()->GetMainWindow ();
+  history_window = GnomeMeeting::Process ()->GetHistoryWindow ();
+  
 
+  PWaitAndSignal m(quit_mutex);
   
   /* Remove the current Gatekeeper */
-  gnomemeeting_threads_enter ();
   gatekeeper = endpoint->GetGatekeeper ();
   if (gatekeeper) {
 
     gk_name = gatekeeper->GetName ();
     msg = g_strdup_printf (_("Unregistered from gatekeeper %s"),
 			   (const char *) gk_name);
-    gnomemeeting_log_insert (gw->history_text_view, msg);
-    gnomemeeting_statusbar_flash (gw->statusbar, msg);
+    gnomemeeting_threads_enter ();
+    gm_history_window_insert (history_window, msg);
+    gm_main_window_flash_message (main_window, msg);
     g_free (msg);
+    gnomemeeting_threads_leave ();
   }
-  gnomemeeting_threads_leave ();
   endpoint->RemoveGatekeeper (0);  
   endpoint->SetUserNameAndAlias (); 
 
@@ -140,18 +145,18 @@ void GMH323Gatekeeper::Main ()
   if (registering_method == 1 && gk_host.IsEmpty ()) {
   
     gnomemeeting_threads_enter ();
-    gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Invalid gatekeeper hostname"), _("Please provide a hostname to use for the gatekeeper."));
+    gnomemeeting_error_dialog (GTK_WINDOW (main_window), _("Invalid gatekeeper hostname"), _("Please provide a hostname to use for the gatekeeper."));
     gnomemeeting_threads_leave ();
 
-    no_error = FALSE;
+    return;
   }
   else if (registering_method == 2 && gk_id.IsEmpty ()) {
 
     gnomemeeting_threads_enter ();
-    gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Invalid gatekeeper ID"), _("Please provide a valid ID for the gatekeeper."));
+    gnomemeeting_error_dialog (GTK_WINDOW (main_window), _("Invalid gatekeeper ID"), _("Please provide a valid ID for the gatekeeper."));
     gnomemeeting_threads_leave ();
 
-    no_error = FALSE;
+    return;
   }
   else {
 
@@ -182,45 +187,40 @@ void GMH323Gatekeeper::Main ()
   
   /* There was an error (missing parameter or registration failed)
      or the user chose to not register */
-  if (!no_error || registering_method == 0) {
+  if (!no_error) {
       
     /* Registering failed */
-    gnomemeeting_threads_enter ();
+    gatekeeper = endpoint->GetGatekeeper ();
+    if (gatekeeper) {
 
-    if (!no_error) {
+      switch (gatekeeper->GetRegistrationFailReason()) {
 
-      gatekeeper = endpoint->GetGatekeeper ();
-      if (gatekeeper) {
-
-	switch (gatekeeper->GetRegistrationFailReason()) {
-
-	case H323Gatekeeper::DuplicateAlias :
-	  msg = g_strdup (_("Another user already exists with the same alias, please use another alias."));
-	  break;
-	case H323Gatekeeper::SecurityDenied :
-	  msg = g_strdup (_("You are not allowed to register to the gatekeeper. Please check your login, password and firewall."));
-	  break;
-	case H323Gatekeeper::TransportError :
-	  msg = g_strdup (_("There was a transport error."));
-	  break;
-	default :
-	  msg = g_strdup (_("No gatekeeper corresponding to your options has been found."));
-	  break;
-	}
+      case H323Gatekeeper::DuplicateAlias :
+	msg = g_strdup (_("Gatekeeper registration failed: duplicate alias"));
+	break;
+      case H323Gatekeeper::SecurityDenied :
+	msg = 
+	  g_strdup (_("Gatekeeper registration failed: bad login/password"));
+	break;
+      case H323Gatekeeper::TransportError :
+	msg = g_strdup (_("Gatekeeper registration failed: transport error"));
+	break;
+      default :
+	msg = g_strdup (_("Gatekeeper registration failed"));
+	break;
       }
-      else
-	msg = g_strdup (_("No gatekeeper corresponding to your options has been found."));
-
-      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Error while registering with gatekeeper"), msg);
-      gnomemeeting_log_insert (gw->history_text_view, msg);
-      g_free (msg);
     }
-    
-    gconf_set_int (H323_GATEKEEPER_KEY "registering_method", 0);
+    else
+      msg = g_strdup (_("Gatekeeper registration failed"));
+
+    gnomemeeting_threads_enter ();
+    gm_main_window_push_message (main_window, msg);
+    gm_history_window_insert (history_window, msg);
+    g_free (msg);
     gnomemeeting_threads_leave ();
   }
   /* Registering is ok */
-  else {
+  else if (registering_method != 0) {
 
     gatekeeper = endpoint->GetGatekeeper ();
     if (gatekeeper)
@@ -229,8 +229,8 @@ void GMH323Gatekeeper::Main ()
       g_strdup_printf (_("Gatekeeper set to %s"), (const char *) gk_name);
     
     gnomemeeting_threads_enter ();
-    gnomemeeting_log_insert (gw->history_text_view, msg);
-    gnomemeeting_statusbar_flash (gw->statusbar, msg);
+    gm_history_window_insert (history_window, msg);
+    gm_main_window_flash_message (main_window, msg);
     gnomemeeting_threads_leave ();
       
     g_free (msg);
@@ -384,7 +384,7 @@ BOOL H323GatekeeperWithNAT::MakeRequest (Request &request)
 	PTRACE(1, "GK\tSwitch to altGK " << gatekeeperIdentifier << " with id " << endpointIdentifier << " successful");
 	if (request.requestPDU.GetChoice ().GetTag() == H323RasPDU::e_admissionRequest) {
 
-	  H225_AdmissionRequest & arq = dynamic_cast<H225_RasMessage&>(request.requestPDU.GetChoice());
+          H225_AdmissionRequest & arq = (H225_AdmissionRequest &) request.requestPDU.GetChoice ();
 
 	  arq.m_gatekeeperIdentifier = gatekeeperIdentifier;
 	  arq.m_endpointIdentifier = endpointIdentifier;
