@@ -73,53 +73,6 @@ gnomemeeting_button_new (const char *,
 			 GtkWidget *);
 
 
-/* DESCRIPTION  :  /
- * BEHAVIOR     :  Displays the gnomemeeting logo in the drawing area.
- * PRE          :  The GtkImage where to put the logo (pixbuf).
- */
-void 
-gnomemeeting_init_main_window_logo (GtkWidget *);
-
-
-/* DESCRIPTION   :  /
- * BEHAVIOR      : Creates a new incoming call popup and returns it.
- * PRE           : The name; and the app UTF-8 char * and the remote URL
- */
-GtkWidget * 
-gnomemeeting_incoming_call_popup_new (gchar *,
-				      gchar *,
-				      gchar *);
-
-
-/* DESCRIPTION   :  /
- * BEHAVIOR      : Flashes a message on the statusbar during a few seconds.
- *                 Removes the previous message.
- * PRE           : The GnomeApp, followed by printf syntax format.
- */
-void 
-gnomemeeting_statusbar_flash (GtkWidget *, const char *, ...);
-
-
-/* DESCRIPTION   :  /
- * BEHAVIOR      : Displays a message on the statusbar or clears it if msg = 0.
- *                 Removes the previous message.
- * PRE           : The GnomeApp, followed by printf syntax format.
- */
-void 
-gnomemeeting_statusbar_push (GtkWidget *, const char *, ...);
-
-
-/* DESCRIPTION   :  /
- * BEHAVIOR      : Creates a video window.
- * PRE           : The title of the window, the drawing area and the window
- *                 name that will be used by gnomemeeting_window_show/hide.
- */
-GtkWidget *
-gnomemeeting_video_window_new (gchar *,
-			       GtkWidget *&,
-			       gchar *);
-
-
 /* DESCRIPTION   :  /
  * BEHAVIOR      : Takes a PString and returns the Left part before a [ or a (.
  * PRE           : An non-empty PString.
@@ -144,10 +97,19 @@ gdk_window_set_always_on_top (GdkWindow *window,
 			      gboolean enable);
 
 
+
+/* DESCRIPTION  :  / 
+ * BEHAVIOR     :  Returns TRUE if the specified window is present and visible
+ *                 on the current workspace, FALSE otherwise.
+ * PRE          :  Argument is a GtkWindow *.
+ */
+gboolean gnomemeeting_window_is_visible (GtkWidget *);
+        
+
 /* DESCRIPTION  :  This callback is called when a window of gnomemeeting
  *                 (addressbook, prefs, ...) has to be shown.
- * BEHAVIOR     :  Restore its size (if applicable) and position from the GConf
- *                 database. The window is given as gpointer.
+ * BEHAVIOR     :  Restore its size (if applicable) and position from the
+ *                 config database. The window is given as gpointer.
  *                 The category can be addressbook, main_window, prefs_window,
  *                 or anything under the
  *                 /apps/gnomemeeting/general/user_interface/ key and is given
@@ -160,7 +122,7 @@ void gnomemeeting_window_show (GtkWidget *);
 
 /* DESCRIPTION  :  This callback is called when a window of gnomemeeting
  *                 (addressbook, prefs, ...) has to be hidden.
- * BEHAVIOR     :  Saves its size (if applicable) and position in the GConf
+ * BEHAVIOR     :  Saves its size (if applicable) and position in the config
  *                 database. The window is given as gpointer.
  *                 The category can be addressbook, main_window, prefs_window,
  *                 or anything under the
